/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * PROPRIETARY/CONFIDENTIAL.  Use of this product is subject to license terms.
 */

package com.sun.portal.search.db;

import com.sun.portal.search.rdm.*;
import com.sun.portal.search.soif.*;
import com.sun.portal.search.util.*;
import com.sun.portal.search.demo.*;

import java.util.*;
import java.io.*;


/**
 *
 * Search engine interface implementation for searching a remote RDM server
 *
 */
public class RemoteRDMDb implements RDMDb {

    String rdmserverurl;
    String rdmserverdb;
    String defaultView = "score,url,hl-url,hl-description,title,hl-title,classification,hl-classification";


    /**
     * Store - creates a transaction for atomic indexing, if none supplied.
     * @param st Token for search
     * @param insoif SOIF document
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public void store(SToken st, SOIF insoif, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("RemoteRDMDb.store(): not implemented");
    }


    /**
     * Empty the database
     * @param st Token for search
     * @param t RDM transaction handle
     * @throws RDMException
     */
    public int purge(SToken st, RDMTransaction t) throws RDMException {
	throw new RDMException("RemoteRDMDb.purge(): not implemented");
    }


    /**
     * Open a database
     * @param st Token for search
     * @param rootdir Database home dir
     * @param dbname Name of database from root database
     * @param rw RDMDb.WRITER or RDMDb.WRCREAT or RDMDb.READER
     * @param mode Unix mode
     * @throws RDMException
     */
    public void open(SToken st, String rootdir, String dbname, int rw, int mode) throws RDMException {
	SOIF dbsoif = DbManager.getRootDbEntry(st, dbname);
	rdmserverurl = dbsoif.getValue("rdmserverurl");
	rdmserverdb = dbsoif.getValue("databasename");
	String defview = dbsoif.getValue("defaultView");
        if (defview != null)
            defaultView = defview;
    }


    /**
     * Close database and index extents
     * @param st Token for search
     * @throws RDMException
     */
    public void close(SToken st) throws RDMException {
	;//do nothing
    }


    /**
     * Obtain the name for the search engine interface
     * @return the name of the search engine activity
     */
    public String getName() {
	return "RemoteRDM Search";
    }


    /**
     * Submit the query to the search engine
     * @param st Token for search
     * @param queryString Search query
     * @param numHits Number of hits
     * @param view Set of attributes to be retrieved
     * @param sortOrder Order for sorting the results
     * @param t RDM transaction handle
     * @return RDMResultSet
     * @throws RDMException
     */
    public RDMResultSet search(SToken st, String query, int numHits, Set view, String sortOrder, RDMTransaction t) throws RDMException {

	Search s;

	try {
	    String csview = "";

	    if (view != null) {
		for (Iterator i = view.iterator(); i.hasNext(); ) {
		    csview = csview + (String)i.next();
		    if (i.hasNext())
			csview += ",";
		}
	    } else {
	      // view can be null even if the client sets a view, because
	      // it is not always passed down (in the case of rdmgr), rather,
	      // it is applied after results are pulled from the databse - 
	      // we need to change that and always do it down here in the
	      // db class. Here we will set a default view to avoid retrieving
	      // everything from the remote server. This view can be configured
	      // in the federated db.
                csview = defaultView;
	    }

	    if (csview.equals(""))
                csview = null;

	    s = new Search(query, csview, sortOrder, 1, numHits, "search", rdmserverdb, rdmserverurl, null);
	    s.doQuery();
	}
	catch (Exception e) {
	    throw new RDMException(e);
	}

	RemoteRDMResultSet rs = new RemoteRDMResultSet(st, this, query, t);
	rs.setRemoteRDMSearchResult(s);
	return rs;

    }


    /**
     * Get search count
     * @return number of docs.
     */
    public int count(SToken st, RDMTransaction t) throws RDMException {
	return 250000000;
    }


    /**
     * Delete RD from database
     * @param st Token for search
     * @param s SOIF document
     * @param view Set of attributes
     * @param flags Options
     * @param t RDM transaction handle
     * @throws RDMException
     */
    public void delete(SToken st, SOIF s, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("RemoteRDMDb.delete() not implemented");
    }


    /**
     * Update an RD with attributes set in View
     * @param st Token for search
     * @param insoif SOIF document
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public void update(SToken st, SOIF insoif, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("RemoteRDMDb.update() not implemented");
    }


    /**
     * Retrieve RD from database, filtered by view
     * @param st Token for search
     * @param url URL string
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public SOIF fetch(SToken st, String url, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("RemoteRDMDb.fetch() not implemented");
    }


    /**
     * Optimize the db
     * @param st Token for search
     * @throws RDMException
     */
    public void optimize(SToken st) throws RDMException {
	throw new RDMException("RemoteRDMDb.optimize() not implemented");
    }


    /**
     * Batch Index
     * @param st Token for search
     * @throws RDMException
     */
    public void indexBatch(SToken st) throws RDMException {
	throw new RDMException("RemoteRDMDb.indexBatch() not implemented");
    }


    /**
     * Recover the db - must be run stand alone (ie, no one else has the db open)
     * @param st Token for search
     * @param dbhome Database home dir
     * @param fatal Option
     * @throws RDMException
     */
    public void recover(SToken st, String dbhome, boolean fatal) throws RDMException {
	throw new RDMException("RemoteRDMDb.recover() not implemented");
    }


}
